
**{{texts.parameters}}**

<table>
    <thead>
        <tr>
            <th>{{texts.parameter}}</th>
            <th>{{texts.parameter_location}}</th>
            <th>{{texts.type}}</th>
            <th>{{texts.default}}</th>
            <th>{{texts.nullable}}</th>
            <th>{{texts.description}}</th>
        </tr>
    </thead>
    <tbody>
{%- for param in parameters %}
        <tr>
            <td class="parameter-name"><code>{{param.name}}</code></td>
            <td>{{param.in}}</td>
            <td>{{read_dict(param, "schema", "type")}}</td>
            <td>{{read_dict(param, "schema", "default", default="")}}</td>
            <td>{{texts.get_yes_no(read_dict(param, "schema", "nullable", default=False))}}</td>
            <td>{{read_dict(param, "description", default="")}}</td>
        </tr>
        {%- endfor %}
    </tbody>
</table>
