:root {
    --bg-color: rgb(17 24 39);
    --text-color: #f5f5f5;
}

@font-face {
    font-family: 'Source Code Pro';
    src: url('../fonts/SourceCodePro-Regular.ttf') format('truetype');
    font-weight: 400;
    font-style: normal;
}

@font-face {
    font-family: 'Source Code Pro';
    src: url('../fonts/SourceCodePro-Bold.ttf') format('truetype');
    font-weight: 700;
    font-style: normal;
}

@font-face {
    font-family: 'Montserrat';
    src: url('../fonts/Montserrat-Regular.ttf') format('truetype');
    font-weight: 400;
    font-style: normal;
}

@font-face {
    font-family: 'Montserrat';
    src: url('../fonts/Montserrat-Bold.ttf') format('truetype');
    font-weight: 700;
    font-style: normal;
}

body, html {
    height: 100%;
    margin: 0;
    display: flex;
    justify-content: center;
    align-items: center;
    font-family: Montserrat, 'Source Code Pro', 'Source Sans 3', -apple-system, monospace;
    text-rendering: optimizeLegibility;
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
    line-height: 1.5;
    background-color: var(--bg-color);
    color: var(--text-color);
    transition: background-color 0.3s ease, color 0.3s ease;
}

.centered-content {
    text-align: center;
    padding: 20px;
    border-radius: 4px;
    display: flex;
    flex-direction: column;
    align-items: center;
    row-gap: 20px;

}

.content-details {
    max-width: 600px;
    text-align: center;
    display: flex;
    flex-direction: column;
    flex: 1;
    width: 100%;
}

.resizable-image {
    max-width: 100%;
    height: auto;
    display: block;
    margin: 0 auto;
}